<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\Util\Functions;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;

class RelRentProdResOpController {    
    protected $RelRentProdResOpService;
    protected $utilService;

    public function __construct(Services\RelRentProdResOpService $RelRentProdResOpService, Utilities\Util $utilService) {
        $this->RelRentProdResOpService = $RelRentProdResOpService;
        $this->utilService = $utilService;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        set_time_limit(120);
        $result = [];
        $conditions = $this->utilService->getParamsAsArray($request);
        $conditions = $this->buildParams($conditions);
        
        if(isset($conditions['CDTIPOCUSTO']) && !empty($conditions['CDTIPOCUSTO'])) {
            $result = $this->RelRentProdResOpService->find($conditions);
            
            $this->realizaCalculos($result);
            
            $response->addDataSet(new DataSet("relRentProdResOp", $result));
            
            return $response;
        }
        
        return $result;
    }

    private function buildParams($conditions) {
        if(isset($conditions['CDFILIAL']) && !empty($conditions['CDFILIAL'])) {
            $conditions['CDFILIAL'] = implode('_', $conditions['CDFILIAL']);
            $conditions['TCDFILIAL'] = 'F';
            
            if(!Functions::arrayKeyExists('CDLOJA', $conditions)) {
                $conditions['CDLOJA'] = '';
                $conditions['TCDLOJA'] = 'T';
            } else {
                $conditions['CDLOJA'] = implode('_', $conditions['CDLOJA']);
                $conditions['TCDLOJA'] = '';
            }

            if(Functions::arrayKeyExists('MESDATA', $conditions) && Functions::arrayKeyExists('ANODATA', $conditions)) {
                $mes = $conditions['MESDATA'];
                $ano = $conditions['ANODATA'];
                $ultimoDia = date('t', strtotime("$ano-$mes-01"));
                
                $conditions['DTINI'] = '01/'.$mes.'/'.$ano;
                $conditions['DTFIN'] = $ultimoDia.'/'.$mes.'/'.$ano;
            }

            if(isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'I') {
                if(empty($conditions['CDARVPRODF'])) {
                    $conditions['CDARVPRODF'] = $conditions['CDARVPRODI'];
                }

                $conditions['TCDPRODUTO'] = null;
                $conditions['CDPRODLIST'] = null;
                $conditions['PRODNIV'] = null;
            } else if(isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'D') {
                $conditions['PRODNIV'] = str_replace('%', '', str_replace('.', '', $conditions['PRODNIV'])).'%';
                $conditions['TCDPRODUTO'] = null;
                $conditions['CDPRODLIST'] = null;
                $conditions['CDARVPRODF'] = null;
                $conditions['CDARVPRODI'] = null;
            } else if(isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'L') {
                $conditions['CDPRODLIST'] = implode('_', $conditions['CDPRODUTO']);
                $conditions['TCDPRODUTO'] = null;
                $conditions['CDARVPRODF'] = null;
                $conditions['CDARVPRODI'] = null;
                $conditions['PRODNIV'] = null;
            } else {
               $conditions['TCDPRODUTO'] = 'T';
               $conditions['CDPRODLIST'] = null;
               $conditions['CDARVPRODF'] = null;
               $conditions['CDARVPRODI'] = null;
               $conditions['PRODNIV'] = null;
            }

            return $conditions;
        }

        return array();
    }
    
    private function realizaCalculos(&$result) {
        $this->calculaCusto($result);
        
        $arrayQtd = $this->getQtdLoja($result);
            
        // calcula a area da loja e a quantidade de funcionarios
        $this->calculaQuantidades($result, $arrayQtd);
        
        $this->calculaGastosLoja($result);
        
        $this->calculaRO($result);
    }
    
    private function calculaCusto(&$result) {
        for($i = 0; $i < count($result); $i++) {
            $result[$i]['LABELFILIAL'] = $result[$i]['CDFILIAL'] . ' - ' . $result[$i]['NMFILIAL'];
            $result[$i]['LABELLOJA']   = $result[$i]['CDLOJA'] . ' - ' . $result[$i]['NMLOJA'];
            
            if($result[$i]['QTPRODVEND'] == 0 || is_null($result[$i]['QTPRODVEND'])) {
                $result[$i]['CUSTOUNIT'] = round(($result[$i]['VRCUSTO'] / 1), 2);
                $aux1 = 1;
            } else {
                $result[$i]['CUSTOUNIT'] = round(($result[$i]['VRCUSTO'] / $result[$i]['QTPRODVEND']), 2);
                $aux1 = $result[$i]['QTPRODVEND'];
            }
            
            if($result[$i]['VRUNITVEND'] == 0) {
                $aux2 = 1;
            } else {
                $aux2 = $result[$i]['VRUNITVEND'];
            }
            
            // calcula o percentual do custo
            $result[$i]['PERCCUS'] = ((($result[$i]['VRCUSTO']) / $aux1) / $aux2) * 100;
        }
    }
    
    private function calculaRO(&$result) {
        $ro = array_reduce($result, function($carry, $item) {
            $qtd      = 1;
            $cdfilial = $item['CDFILIAL'];
            $cdloja   = $item['CDLOJA'];
            
            foreach($carry as &$element) {
                if(($element['CDFILIAL'] == $cdfilial) && ($element['CDLOJA'] == $cdloja)) {
                    $qtd = 0;
                    break;
                }
            }
            
            if($qtd) {
                $carry[] = [
                    'CDFILIAL' => $cdfilial,
                    'CDLOJA'   => $cdloja,
                	'ROTOTAL'  => 0
                ];
            }
            
            return $carry;
        }, []);

        for($i = 0; $i < count($result); $i++) {
            // realiza o calculo de cada RO
            $result[$i]['RO'] = $result[$i]['VRTOTAL'] - $result[$i]['VRIMPOSTO'] - $result[$i]['VRCUSTO'] - $result[$i]['ROYALTIES'] - $result[$i]['FPROPMARK'] - $result[$i]['GASTOSLOJA'] - $result[$i]['OD'];
            
            for($j = 0; $j < count($ro); $j++) {
                // somando os valores totais de cada RO para fazer a regra de 3
                if(($result[$i]['CDFILIAL'] == $ro[$j]['CDFILIAL']) && ($result[$i]['CDLOJA'] == $ro[$j]['CDLOJA'])) {
                    $ro[$j]['ROTOTAL'] += $result[$i]['RO'];
                }
            }
        }

        for($i = 0; $i < count($result); $i++) {
            for($j = 0; $j < count($ro); $j++) {
                if(($result[$i]['CDFILIAL'] == $ro[$j]['CDFILIAL']) && ($result[$i]['CDLOJA'] == $ro[$j]['CDLOJA'])) {
                    $result[$i]['RENTABPERC'] = ($result[$i]['RO'] * 100) / $ro[$j]['ROTOTAL'];
                }
            }
        }
    }
    
    private function getQtdLoja($result) {
        $qtdArray = array_reduce($result, function($carry, $item) {
            $cdfilial = $item['CDFILIAL'];
            $cdloja = $item['CDLOJA'];
            
            if (isset($carry[$cdfilial][$cdloja])) {
                $carry[$cdfilial][$cdloja]++;
            } else {
                $carry[$cdfilial][$cdloja] = 1;
            }
        
            return $carry;
        }, []);
        
        $qtdArray = array_map(function($key) use ($qtdArray) {
            return [
                'CDFILIAL' => $key,
                'CDLOJA'   => array_keys($qtdArray[$key]),
                'QTD'      => array_values($qtdArray[$key])
            ];
        }, array_keys($qtdArray));
        
        return $qtdArray;
    }
    
    private function calculaQuantidades(&$result, $qtd) {
        for($i = 0; $i < count($result); $i++) {
            for($j = 0; $j < count($qtd); $j++) {
                if($result[$i]['CDFILIAL'] == $qtd[$j]['CDFILIAL']) {
                    $result[$i]['M2']  = $result[$i]['VRM2']  / $qtd[$j]['QTD'][0];
                    $result[$i]['QFP'] = $result[$i]['VRQFP'] / $qtd[$j]['QTD'][0];
                    $result[$i]['QFR'] = $result[$i]['VRQFR'] / $qtd[$j]['QTD'][0];
                }
            }
        }
    }
    
    private function calculaGastosLoja(&$result) {
        // pegando as marcas
        $marcas = array_reduce($result, function($carry, $item) {
            $qtd      = 1;
            $marca    = $item['NMPRODUTOAVO'];
            $cdfilial = $item['CDFILIAL'];
            $cdloja   = $item['CDLOJA'];
            
            foreach($carry as &$element) {
                if(($element['MARCA'] == $marca) && ($element['CDFILIAL'] == $cdfilial) && ($element['CDLOJA'] == $cdloja)) {
                    $element['QTD']++;
                    $qtd = 0;
                    break;
                }
            }
            
            if($qtd) {
                $carry[] = [
                    'CDFILIAL'   => $cdfilial,
                    'CDLOJA'     => $cdloja,
                	'MARCA'      => $marca,
                	'VRTOTMARCA' => 0,
                	'VRTOT'      => 0,
                	'QTD'        => 1,
                	'PORC'       => 0
                ];
            }
            
            return $carry;
        }, []);
        
        for($i = 0; $i < count($result); $i++) {
            for($j = 0; $j < count($marcas); $j++) {
                // somando os valores totais de cada marca para fazer a regra de 3
                if($result[$i]['NMPRODUTOAVO'] == $marcas[$j]['MARCA']) {
                    $marcas[$j]['VRTOTMARCA'] += $result[$i]['VRTOTAL'];
                }
                
                // somando os valores totais de todas as marcas de uma unidade e loja especifica
                if(($result[$i]['CDFILIAL'] == $marcas[$j]['CDFILIAL']) && ($result[$i]['CDLOJA'] == $marcas[$j]['CDLOJA'])) {
                   $marcas[$j]['VRTOT'] += $result[$i]['VRTOTAL'];
                }
            }
        }
        
        // pega a porcentagem de cada marca
        for($i = 0; $i < count($result); $i++) {
            for($j = 0; $j < count($marcas); $j++) {
                if(($result[$i]['CDFILIAL'] == $marcas[$j]['CDFILIAL']) && ($result[$i]['CDLOJA'] == $marcas[$j]['CDLOJA']) && ($result[$i]['NMPRODUTOAVO'] == $marcas[$j]['MARCA'])) {
                    $marcas[$j]['PORC'] = ($marcas[$j]['VRTOTMARCA'] * 100) / $marcas[$j]['VRTOT'];
                    
                    $gastosLoja = ($result[$i]['VRGASTOSLOJA'] * $marcas[$j]['PORC']) / 100;
                    $result[$i]['GASTOSLOJA'] = $gastosLoja / $marcas[$j]['QTD'];
                    
                    $od = ($marcas[$j]['VRTOTMARCA'] * $result[$i]['VRODPERC']) / 100;
                    $result[$i]['OD'] = $od / $marcas[$j]['QTD'];
                    
                    $fundoPropMark = ($marcas[$j]['VRTOTMARCA'] * $result[$i]['VRFPROPMARKPERC']) / 100;
                    $result[$i]['FPROPMARK'] = $fundoPropMark / $marcas[$j]['QTD'];
                    
                    $royalties = ($marcas[$j]['VRTOTMARCA'] * $result[$i]['VRROYALTIESPERC']) / 100;
                    $result[$i]['ROYALTIES'] = $royalties / $marcas[$j]['QTD'];
                }
            }
        }
    }
}